use crate::page::Page;

pub struct Pager {
    pages: Vec<Option<Page>>,
    page_size: usize,
}

impl Pager {
    pub fn new(page_size: usize) -> Self {
        Self {
            page_size,
            pages: Vec::new(),
        }
    }

    pub fn get_mut_or_alloc(&mut self, page_num: usize) -> &mut Page {
        if page_num >= self.pages.len() {
            self.grow_pages(page_num)
        }

        if self.pages[page_num].is_none() {
            let buf = vec![0; self.page_size];
            let page = Page::new(buf);
            self.pages[page_num] = Some(page)
        }

        self.pages[page_num].as_mut().unwrap()
    }

    #[inline]
    fn grow_pages(&mut self, index: usize) {
        println!("debug: Growing pages");
        self.pages.resize(index + 1, None)
    }
}
